//*****************************************************************************
//! msp_wm430fr4_golbal.c
//
//!
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************

#include <msp_wm430fr4_rtc.h>
// uint8_t const ui8Month_Table[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
//tWMRTC tRTCData;

//*****************************************************************************
//
//! Perform GPIO initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void GPIO_Config(void)
{
	P1OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
	P1DIR |= BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;
	P2OUT &= ~(BIT4 + BIT5 + BIT6 + BIT7);
	P2DIR |= BIT4 + BIT5 + BIT6 + BIT7;
	P4OUT &= ~BIT0;
	P4DIR |= BIT0;
	P5OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
	P5DIR |= BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;
	P8OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3);
	P8DIR |= BIT0 + BIT1 + BIT2 + BIT3;
}

//*****************************************************************************
//
//! Perform System clock initialize.
//! ACLK = XT1CLK = 32.738kHz  MCLK = SMCLK = default ~ 1.04MHz
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void CLOCK_Config()
{
	P4SEL0 |= BIT1 + BIT2;                  // Select pin4.1 pin4.2 as crystal function
	do
	{
		CSCTL7 &= ~(XT1OFFG + DCOFFG);    	// Clear XT1 and DCO fault flag
		SFRIFG1 &= ~OFIFG;
	}while (SFRIFG1 & OFIFG);             	// Test oscillator fault flag

	CSCTL4 = SELA__XT1CLK;					// IMPORTANT. Select ACLK = XT1 = 32768Hz
}

//*****************************************************************************
//
//! Perform RTC initialize.
//! Clock source XT1CLK
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void RTC_Config()
{
	RTCCTL = RTCSS__XT1CLK + RTCSR + RTCPS__1 | RTCIE;
	RTCMOD = 32768;		             //1s
	tRTCData.ui8Year = 15;              // RTC initialize
	tRTCData.ui8Month = 3;
	tRTCData.ui8Day = 1;
	tRTCData.ui8Hour = 10;
	tRTCData.ui8Minute = 15;
	tRTCData.ui8Second = 00;

}
//*****************************************************************************
//
//! Perform ADC initialize.
//! Clock source MODCLK / Channel A13 Internal reference 1.5V
//! Reference AVCC+ AVSS- / 50ksps
//! 10-bit resolution
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void ADC10_Config()
{
	CSCTL8 |= MODCLKREQEN;
	PMMCTL0 = SVSHE + PMMPW;
	PMMCTL2 |= INTREFEN;                // 1.5V voltage reference generate
	ADCCTL0 = ADCSHT_8 + ADCON;         // 256 ADCCLK cycles
	ADCCTL1 = ADCSHS_0 + ADCSHP + ADCSSEL_0 + ADCCONSEQ_0;  // ADCSC bit SHS,
	// MODCLK
	// SC-SC
	ADCCTL2 |= ADCRES + ADCSR;          // 10 bit resolution, 50ksps
	ADCMCTL0 = ADCSREF_0 + ADCINCH_13;  // Reference AVCC+/AVSS-, Channel 13, REF voltage
	ADCIE |= ADCIE0;                    // Interrupt enable

//	__delay_cycles(400); //Original with MCLK=1MHz
	__delay_cycles(800); // assumes MCLK=2MHz
}

//*****************************************************************************
//
//! Perform Timer configuration.
//!
//
//! \param
//!
//! \return None.
//

//*****************************************************************************
//
//! Perform Water Meter board initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void WM_Initialize(void)
{
	WDTCTL = WDTPW + WDTHOLD;  	// Stop watchdog
	//	WDTCTL = WDTPW + WDTCNTCL + WDTSSEL0 + WDTIS0 + WDTIS1;  // Set watchdog 16s

	PM5CTL0 &= ~LOCKLPM5;      	// Disable the GPIO power-on default high-impedance mode
								// to activate previously configured port settings
	GPIO_Config();              // GPIO initialize
	LED1_ON();

	CLOCK_Config();             // System clock initialize

	RTC_Config();
	ADC10_Config();
	P1SEL0 |= BIT5;
	P1REN &= ~BIT5; // to be commented
	TA0CTL = 0;
	LED1_OFF();
}

void FRAMWrite_Long(uint32_t *pFRAM_write_ptr, uint32_t ui32data)
{
	SYSCFG0 &= ~PFWP;
	*pFRAM_write_ptr = ui32data;
	SYSCFG0 |= PFWP;
}

void FRAMWrite_Char(uint8_t *pFRAM_write_ptr, uint8_t ui8data)
{
	SYSCFG0 &= ~PFWP;
	*pFRAM_write_ptr = ui8data;
	SYSCFG0 |= PFWP;
}
